@@ -6,7 +6,7 @@ use warnings FATAL => 'all';
 use parent qw(Exporter);
 use Carp qw(croak);
 
-our @EXPORT_OK = qw(spath);
+our @EXPORT_OK = qw(slist spath);
 
 =head1 NAME
 
@@ -14,11 +14,11 @@ Struct::Path - Path for nested structures where path is also a structure
 
 =head1 VERSION
 
-Version 0.21
+Version 0.32
 
 =cut
 
-our $VERSION = '0.21';
+our $VERSION = '0.32';
 
 =head1 SYNOPSIS
 
@@ -27,64 +27,122 @@ our $VERSION = '0.21';
     $s = [
         0,
         1,
-        {2a => {2aa => '2aav', 2ab => '2abv'}},
+        {'2a' => {'2aa' => '2aav', '2ab' => '2abv'}},
         undef
     ];
 
-    @r = spath($s, [ [3,0,1] ]);
+    @list = slist($s);                              # get all paths and their values
+    # @list == (
+    #    [[[0]],0],
+    #    [[[1]],1],
+    #    [[[2],{keys => ['2a']},{keys => ['2aa']}],'2aav'],
+    #    [[[2],{keys => ['2a']},{keys => ['2ab']}],'2abv'],
+    #    [[[3]],undef]
+    # )
+
+    @r = spath($s, [ [3,0,1] ]);                    # get refs to values by paths
     # @r == (\undef, \0, \1)
 
-    @r = spath($s, [ [2],{keys => ['2a']},{} ]);
-    # @r == (\2aav, \2abv)
+    @r = spath($s, [ [2],{keys => ['2a']},{} ]);    # same, another example
+    # @r == (\'2aav', \'2abv')
 
-    ${$r[0]} =~ s/2a/blah-blah-/;
-    # $s->[2]{2a}{2aa} == "blah-blah-av"
+    ${$r[0]} =~ s/2a/blah-blah-/;                   # replace substructire by path
+    # $s->[2]{2a}{2aa} eq "blah-blah-av"
 
 =head1 EXPORT
 
-Nothing exports by default.
+Nothing is exported by default.
 
 =head1 SUBROUTINES
 
+=head2 slist
+
+Returns list of paths and their values from structure.
+
+    @list = slist($struct, %opts)
+
+=head3 Available options
+
+=over 4
+
+=item depth C<< <N> >>
+
+Don't dive into structure deeper than defined level.
+
+=back
+
+=cut
+
+sub slist($;@) {
+    my ($struct, %opts) = @_;
+    my @list = ([[], $struct]); # init: [path, lastref]
+
+    my $continue = 1;
+    my $depth = 0;
+    while ($continue) {
+        last if (defined $opts{depth} and $depth >= $opts{depth});
+        $continue = 0;
+        my @new;
+        while (my $path = shift @list) {
+            if (ref $path->[1] eq 'ARRAY' and @{$path->[1]}) {
+                for (my $i = 0; $i < @{$path->[1]}; $i++) {
+                    push @new, [[@{$path->[0]}, [$i]], $path->[1]->[$i]];
+                }
+                $continue = 1;
+            } elsif (ref $path->[1] eq 'HASH' and keys %{$path->[1]}) {
+                for my $k (sort keys %{$path->[1]}) {
+                    push @new, [[@{$path->[0]}, {keys => [$k]}], $path->[1]->{$k}];
+                }
+                $continue = 1;
+            } else {
+                push @new, $path; # complete path
+            }
+        }
+        @list = @new;
+        $depth++;
+    }
+
+    return @list;
+}
+
 =head2 spath
 
 Returns list of references from structure.
 
     @list = spath($struct, $path, %opts)
 
-=head3 Addressing scheme
+=head3 Addressing method
 
-It's simple: path is a list of 'steps', each represents nested level in passed structure. Arrayref as a step
+It's simple: path is a list of 'steps', each represents nested level in structure. Arrayref as a step
 stands for ARRAY in structure and must contain desired indexes or be empty (means "all items"). Sequence for indexes
 is important and defines result sequence. Almost the same for HASHES - step must be a hashref, must contain key
 C<keys> which value must contain list of desired keys in structure or may be empty (all keys). Sequence
 in C<keys> list defines result sequence.
 
-So, different combinations of steps allows to reach different parts of structure.
+Why existed *Path* libs (L</"SEE ALSO">) not enough?
+This scheme has no collisions for paths like '/a/0/c' ('0' may be an array index or a key for hash, depends on passed
+structure). In some cases this is important, for example, when you want to define exact path in structure, but
+unable to validate it's schema.
 
-Weird? Why this needed?
-First: this addressing methos is mashine friendly. Second: it allows to specify exact address in structure
-without hardcoding it.
-
-See L<Struct::Path::PerlStyle> if you're like this approach, but interested in human friendly path definition method.
+See L<Struct::Path::PerlStyle> if you're looking for human friendly path definition method.
 
 =head3 Available options
 
 =over 4
 
-=item delete
+=item delete C<< <true|false> >>
 
-Delete specified by path items from structure if set to true value.
+Delete specified by path items from structure.
 
-=item deref
+=item deref C<< <true|false> >>
 
-Dereference result items if set to some true value.
+Dereference result items.
 
-=item expand
+=item expand C<< <true|false> >>
 
 Expand structure if specified in path items does't exists. All newly created items initialized by C<undef>.
 
-=item strict
+=item strict C<< <true|false> >>
 
 Croak if at least one element, specified in path, absent in the struct.
 
@@ -94,9 +152,8 @@ Croak if at least one element, specified in path, absent in the struct.
 
 sub spath($$;@) {
     my ($struct, $path, %opts) = @_;
-    croak "Stuct must be reference to ARRAY or HASH" unless (ref $struct eq 'ARRAY' or ref $struct eq 'HASH');
     croak "Path must be arrayref" unless (ref $path eq 'ARRAY');
-    my $refs = [ \$struct ]; # init
+    my $refs = (ref $struct eq 'ARRAY' or ref $struct eq 'HASH' or not ref $struct) ? [ \$struct ] : [ $struct ]; #init
 
     my $sc = 0; # step counter
     for my $step (@{$path}) {
@@ -110,6 +167,7 @@ sub spath($$;@) {
                         next;
                     }
                 }
+
                 if (@{$step}) {
                     for my $i (@{$step}) {
                         unless ($opts{expand} or @{${$r}} > $i) {
@@ -210,7 +268,7 @@ L<http://search.cpan.org/dist/Struct-Path/>
 L<Data::Diver> L<Data::DPath> L<Data::DRef> L<Data::Focus> L<Data::Hierarchy> L<Data::Nested> L<Data::PathSimple>
 L<Data::Reach> L<Data::Spath> L<JSON::Path> L<MarpaX::xPathLike> L<Sereal::Path>
 
-L<Struct::Path::PerlStyle> L<Struct::Diff>
+L<Struct::Diff> L<Struct::Path::PerlStyle>
 
 =head1 LICENSE AND COPYRIGHT
 
